@@ -1,8 +1,8 @@
 /*
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2015 Alessandro Bono
- * Copyright © 2014, 2015 Pranav Kant
- * Copyright © 2012, 2013, 2014, 2015, 2016 Red Hat, Inc.
+ * Copyright © 2014 – 2015 Pranav Kant
+ * Copyright © 2012 – 2016 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,6 +52,7 @@
 #include "photos-search-match.h"
 #include "photos-search-type.h"
 #include "photos-search-provider.h"
+#include "photos-selection-controller.h"
 #include "photos-single-item-job.h"
 #include "photos-source.h"
 #include "photos-source-manager.h"
@@ -70,6 +71,7 @@ struct _PhotosApplicationPrivate
   GSettings *ss_settings;
   GSimpleAction *brightness_contrast_action;
   GSimpleAction *crop_action;
+  GSimpleAction *delete_action;
   GSimpleAction *denoise_action;
   GSimpleAction *edit_action;
   GSimpleAction *edit_cancel_action;
@@ -98,6 +100,7 @@ struct _PhotosApplicationPrivate
   PhotosCameraCache *camera_cache;
   PhotosSearchContextState *state;
   PhotosSearchProvider *search_provider;
+  PhotosSelectionController *sel_cntrlr;
   TrackerExtractPriority *extract_priority;
   guint create_miners_count;
   guint32 activation_timestamp;
@@ -259,18 +262,53 @@ photos_application_action_toggle (GSimpleAction *simple, GVariant *parameter, gp
 }
 
 
+static PhotosBaseItem *
+photos_application_get_selection_or_active_item (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  PhotosBaseItem *item = NULL;
+
+  if (photos_selection_controller_get_selection_mode (priv->sel_cntrlr))
+    {
+      GList *selection;
+      const gchar *urn;
+
+      selection = photos_selection_controller_get_selection (priv->sel_cntrlr);
+      if (selection == NULL || selection->next != NULL) /* length != 1 */
+        goto out;
+
+      urn = (gchar *) selection->data;
+      item = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (priv->state->item_mngr, urn));
+    }
+  else
+    {
+      item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+    }
+
+ out:
+  return item;
+}
+
+
 static void
 photos_application_actions_update (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
   PhotosBaseItem *item;
+  GList *l;
+  GList *selection;
   PhotosLoadState load_state;
   PhotosWindowMode mode;
+  gboolean can_open;
+  gboolean can_trash;
   gboolean enable;
+  gboolean selection_mode;
 
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  item = photos_application_get_selection_or_active_item (self);
   load_state = photos_item_manager_get_load_state (priv->state->item_mngr);
   mode = photos_mode_controller_get_window_mode (priv->state->mode_cntrlr);
+  selection = photos_selection_controller_get_selection (priv->sel_cntrlr);
+  selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
 
   enable = (mode == PHOTOS_WINDOW_MODE_EDIT);
   g_simple_action_set_enabled (priv->brightness_contrast_action, enable);
@@ -303,13 +341,39 @@ photos_application_actions_update (PhotosApplication *self)
 
   enable = (load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW);
   g_simple_action_set_enabled (priv->gear_action, enable);
-  g_simple_action_set_enabled (priv->open_action, enable);
-  g_simple_action_set_enabled (priv->print_action, enable);
   g_simple_action_set_enabled (priv->properties_action, enable);
-  g_simple_action_set_enabled (priv->save_action, enable);
   g_simple_action_set_enabled (priv->set_bg_action, enable);
   g_simple_action_set_enabled (priv->set_ss_action, enable);
 
+  enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW)
+            || (selection_mode && item != NULL && !photos_base_item_is_collection (item)));
+  g_simple_action_set_enabled (priv->print_action, enable);
+  g_simple_action_set_enabled (priv->save_action, enable);
+
+  can_open = FALSE;
+  can_trash = selection != NULL;
+  for (l = selection; l != NULL; l = l->next)
+    {
+      PhotosBaseItem *selected_item;
+      const gchar *urn = (gchar *) l->data;
+
+      selected_item = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (priv->state->item_mngr, urn));
+      can_trash = can_trash && photos_base_item_can_trash (selected_item);
+
+      if (photos_base_item_get_default_app_name (selected_item) != NULL)
+        can_open = TRUE;
+    }
+
+  enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED
+             && mode == PHOTOS_WINDOW_MODE_PREVIEW
+             && photos_base_item_can_trash (item))
+            || (selection_mode && can_trash));
+  g_simple_action_set_enabled (priv->delete_action, enable);
+
+  enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW)
+            || (selection_mode && can_open));
+  g_simple_action_set_enabled (priv->open_action, enable);
+
   enable = (load_state == PHOTOS_LOAD_STATE_FINISHED
             && mode == PHOTOS_WINDOW_MODE_PREVIEW
             && photos_base_item_can_edit (item));
@@ -668,42 +732,6 @@ photos_application_edit_cancel (PhotosApplication *self)
 }
 
 
-static void
-photos_application_edit_done_pipeline_save (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
-  PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
-  GError *error;
-
-  error = NULL;
-  if (!photos_base_item_pipeline_save_finish (item, res, &error))
-    {
-      g_warning ("Unable to save pipeline: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
-
-  photos_mode_controller_go_back (self->priv->state->mode_cntrlr);
-
- out:
-  g_application_release (G_APPLICATION (self));
-}
-
-
-static void
-photos_application_edit_done (PhotosApplication *self)
-{
-  PhotosApplicationPrivate *priv = self->priv;
-  PhotosBaseItem *item;
-
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
-  g_return_if_fail (item != NULL);
-
-  g_application_hold (G_APPLICATION (self));
-  photos_base_item_pipeline_save_async (item, NULL, photos_application_edit_done_pipeline_save, self);
-}
-
-
 static void
 photos_application_edit_current (PhotosApplication *self)
 {
@@ -711,8 +739,7 @@ photos_application_edit_current (PhotosApplication *self)
   PhotosBaseItem *item;
 
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
-  if (item == NULL)
-    return;
+  g_return_if_fail (item != NULL);
 
   photos_mode_controller_set_window_mode (priv->state->mode_cntrlr, PHOTOS_WINDOW_MODE_EDIT);
 }
@@ -774,13 +801,32 @@ photos_application_open_current (PhotosApplication *self)
   PhotosBaseItem *item;
   guint32 time;
 
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
-  if (item == NULL)
-    return;
-
   screen = gtk_window_get_screen (GTK_WINDOW (priv->main_window));
   time = gtk_get_current_event_time ();
-  photos_base_item_open (item, screen, time);
+
+  if (photos_selection_controller_get_selection_mode (priv->sel_cntrlr))
+    {
+      GList *l;
+      GList *selection;
+
+      selection = photos_selection_controller_get_selection (priv->sel_cntrlr);
+      for (l = selection; l != NULL; l = l->next)
+        {
+          const gchar *urn = (gchar *) l->data;
+
+          item = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (priv->state->item_mngr, urn));
+          photos_base_item_open (item, screen, time);
+        }
+    }
+  else
+    {
+      item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+      g_return_if_fail (item != NULL);
+
+      photos_base_item_open (item, screen, time);
+    }
+
+  photos_selection_controller_set_selection_mode (priv->sel_cntrlr, FALSE);
 }
 
 
@@ -790,9 +836,8 @@ photos_application_print_current (PhotosApplication *self)
   PhotosApplicationPrivate *priv = self->priv;
   PhotosBaseItem *item;
 
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
-  if (item == NULL)
-    return;
+  item = photos_application_get_selection_or_active_item (self);
+  g_return_if_fail (item != NULL);
 
   photos_base_item_print (item, priv->main_window);
 }
@@ -807,8 +852,7 @@ photos_application_properties (PhotosApplication *self)
   const gchar *id;
 
   item = photos_base_manager_get_active_object (priv->state->item_mngr);
-  if (item == NULL)
-    return;
+  g_return_if_fail (item != NULL);
 
   id = photos_filterable_get_id (PHOTOS_FILTERABLE (item));
   dialog = photos_properties_dialog_new (GTK_WINDOW (priv->main_window), id);
@@ -916,8 +960,7 @@ photos_application_remote_display_current (PhotosApplication *self)
   const gchar *urn;
 
   item = photos_base_manager_get_active_object (priv->state->item_mngr);
-  if (item == NULL)
-    return;
+  g_return_if_fail (item != NULL);
 
   urn = photos_filterable_get_id (PHOTOS_FILTERABLE (item));
   dialog = photos_dlna_renderers_dialog_new (GTK_WINDOW (priv->main_window), urn);
@@ -948,6 +991,7 @@ photos_application_save_save (GObject *source_object, GAsyncResult *res, gpointe
     }
 
  out:
+  photos_selection_controller_set_selection_mode (self->priv->sel_cntrlr, FALSE);
   g_application_release (G_APPLICATION (self));
 }
 
@@ -956,7 +1000,6 @@ static void
 photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer user_data)
 {
   PhotosApplication *self = PHOTOS_APPLICATION (user_data);
-  PhotosApplicationPrivate *priv = self->priv;
   GError *error;
   GFile *export = NULL;
   GFile *tmp;
@@ -969,7 +1012,7 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   if (response_id != GTK_RESPONSE_OK)
     goto out;
 
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  item = photos_application_get_selection_or_active_item (self);
   g_return_if_fail (item != NULL);
 
   pictures_path = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
@@ -1025,7 +1068,7 @@ photos_application_save (PhotosApplication *self)
   GtkWidget *dialog;
   PhotosBaseItem *item;
 
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  item = photos_application_get_selection_or_active_item (self);
   g_return_if_fail (item != NULL);
   g_return_if_fail (!photos_base_item_is_collection (item));
 
@@ -1075,8 +1118,7 @@ photos_application_set_bg_common (PhotosApplication *self, GVariant *parameter,
   PhotosBaseItem *item;
 
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (self->priv->state->item_mngr));
-  if (item == NULL)
-    return;
+  g_return_if_fail (item != NULL);
 
   settings = G_SETTINGS (g_object_get_data (G_OBJECT (action), "settings"));
   photos_base_item_download_async (item, NULL, photos_application_set_bg_common_download, g_object_ref (settings));
@@ -1185,6 +1227,13 @@ photos_application_window_mode_changed (PhotosApplication *self)
 }
 
 
+static void
+photos_application_selection_changed (PhotosApplication *self)
+{
+  photos_application_actions_update (self);
+}
+
+
 static void
 photos_application_activate (GApplication *application)
 {
@@ -1287,10 +1336,12 @@ photos_application_startup (GApplication *application)
   GVariant *state;
   GVariantType *parameter_type;
   const gchar *delete_accels[3] = {"Delete", "KP_Delete", NULL};
+  const gchar *edit_accels[2] = {"<Primary>e", NULL};
   const gchar *fullscreen_accels[2] = {"F11", NULL};
   const gchar *gear_menu_accels[2] = {"F10", NULL};
   const gchar *print_current_accels[2] = {"<Primary>p", NULL};
   const gchar *quit_accels[2] = {"<Primary>q", NULL};
+  const gchar *save_accels[2] = {"<Primary>x", NULL};
   const gchar *search_accels[2] = {"<Primary>f", NULL};
   const gchar *select_all_accels[2] = {"<Primary>a", NULL};
 
@@ -1339,6 +1390,17 @@ photos_application_startup (GApplication *application)
    */
   priv->camera_cache = photos_camera_cache_dup_singleton ();
 
+  priv->sel_cntrlr = photos_selection_controller_dup_singleton ();
+  g_signal_connect_swapped (priv->sel_cntrlr,
+                            "selection-changed",
+                            G_CALLBACK (photos_application_selection_changed),
+                            self);
+
+  g_signal_connect_swapped (priv->sel_cntrlr,
+                            "selection-mode-changed",
+                            G_CALLBACK (photos_application_selection_changed),
+                            self);
+
   action = g_simple_action_new ("about", NULL);
   g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_about), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
@@ -1354,9 +1416,8 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->crop_action));
   g_variant_type_free (parameter_type);
 
-  action = g_simple_action_new ("delete", NULL);
-  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
-  g_object_unref (action);
+  priv->delete_action = g_simple_action_new ("delete", NULL);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->delete_action));
 
   priv->denoise_action = g_simple_action_new ("denoise-current", G_VARIANT_TYPE_UINT16);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->denoise_action));
@@ -1373,10 +1434,6 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->edit_action));
 
   priv->edit_done_action = g_simple_action_new ("edit-done", NULL);
-  g_signal_connect_swapped (priv->edit_done_action,
-                            "activate",
-                            G_CALLBACK (photos_application_edit_done),
-                            self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->edit_done_action));
 
   priv->fs_action = g_simple_action_new ("fullscreen", NULL);
@@ -1494,9 +1551,11 @@ photos_application_startup (GApplication *application)
 
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.quit", quit_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.delete", delete_accels);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.edit-current", edit_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.fullscreen", fullscreen_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.gear-menu", gear_menu_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.print-current", print_current_accels);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.save-current", save_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.search", search_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.select-all", select_all_accels);
 
@@ -1558,6 +1617,7 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->ss_settings);
   g_clear_object (&priv->brightness_contrast_action);
   g_clear_object (&priv->crop_action);
+  g_clear_object (&priv->delete_action);
   g_clear_object (&priv->denoise_action);
   g_clear_object (&priv->edit_action);
   g_clear_object (&priv->edit_cancel_action);
@@ -1583,6 +1643,7 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->sharpen_action);
   g_clear_object (&priv->undo_action);
   g_clear_object (&priv->camera_cache);
+  g_clear_object (&priv->sel_cntrlr);
   g_clear_object (&priv->extract_priority);
 
   if (priv->state != NULL)
