@@ -1,8 +1,8 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015 Red Hat, Inc.
- * Copyright © 2015 Umang Jain
- * Copyright © 2011, 2012, 2013, 2014, 2015 Yorba Foundation
+ * Copyright © 2015 – 2016 Red Hat, Inc.
+ * Copyright © 2015 – 2016 Umang Jain
+ * Copyright © 2011 – 2015 Yorba Foundation
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,8 +35,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "gegl-gtk-view.h"
 #include "photos-icons.h"
+#include "photos-image-view.h"
 #include "photos-tool.h"
 #include "photos-tool-crop.h"
 #include "photos-utils.h"
@@ -62,9 +62,8 @@ struct _PhotosToolCrop
   GAction *crop;
   GeglRectangle bbox_zoomed;
   GeglRectangle bbox_source;
-  GtkListStore *model;
   GtkWidget *box;
-  GtkWidget *combo_box;
+  GtkWidget *list_box;
   GtkWidget *lock_button;
   GtkWidget *reset_button;
   GtkWidget *revealer;
@@ -81,6 +80,7 @@ struct _PhotosToolCrop
   gdouble crop_y;
   gdouble event_x_last;
   gdouble event_y_last;
+  gint list_box_active;
   gulong size_allocate_id;
 };
 
@@ -124,7 +124,7 @@ enum
   CONSTRAINT_COLUMN_BASIS_WIDTH = 3
 };
 
-/* "Free" is excluded from the GtkComboBox and represented by the
+/* "Free" is excluded from the GtkListBox and represented by the
  *  GtkCheckButton. Adjust accordingly.
  */
 static PhotosToolCropConstraint CONSTRAINTS[] =
@@ -132,10 +132,9 @@ static PhotosToolCropConstraint CONSTRAINTS[] =
   { PHOTOS_TOOL_CROP_ASPECT_RATIO_ANY, N_("Free"), 0, 0 },
   { PHOTOS_TOOL_CROP_ASPECT_RATIO_ORIGINAL, N_("Original Size"), 0, 0 },
   { PHOTOS_TOOL_CROP_ASPECT_RATIO_SCREEN, N_("Screen"), 0, 0 },
-  { PHOTOS_TOOL_CROP_ASPECT_RATIO_BASIS, N_("Golden Cut"), 100000, 161803},
+  { PHOTOS_TOOL_CROP_ASPECT_RATIO_BASIS, N_("Golden ratio"), 100000, 161803},
   { PHOTOS_TOOL_CROP_ASPECT_RATIO_BASIS, N_("Square"), 1, 1 },
-  { PHOTOS_TOOL_CROP_ASPECT_RATIO_BASIS, N_("A3 (297 × 420 mm)"), 297, 420 },
-  { PHOTOS_TOOL_CROP_ASPECT_RATIO_BASIS, N_("A4 (210 × 297 mm)"), 297, 210 }
+  { PHOTOS_TOOL_CROP_ASPECT_RATIO_BASIS, N_("A paper sizes"), 297, 420 }
 };
 
 static const gdouble CROP_MIN_SIZE = 16.0;
@@ -214,10 +213,10 @@ photos_tool_crop_redraw_damaged_area (PhotosToolCrop *self)
   gdouble x;
   gdouble y;
 
-  x = (gdouble) gegl_gtk_view_get_x (GEGL_GTK_VIEW (self->view));
+  x = (gdouble) photos_image_view_get_x (PHOTOS_IMAGE_VIEW (self->view));
   x = -x + self->crop_x - damage_offset;
 
-  y = (gdouble) gegl_gtk_view_get_y (GEGL_GTK_VIEW (self->view));
+  y = (gdouble) photos_image_view_get_y (PHOTOS_IMAGE_VIEW (self->view));
   y = -y + self->crop_y - damage_offset;
 
   area.height = (gint) (self->crop_height + 2 * damage_offset + 0.5) + 2;
@@ -240,7 +239,7 @@ photos_tool_crop_surface_create (PhotosToolCrop *self)
   g_clear_pointer (&self->surface, (GDestroyNotify) cairo_surface_destroy);
 
   window = gtk_widget_get_window (self->view);
-  zoom = gegl_gtk_view_get_zoom (GEGL_GTK_VIEW (self->view));
+  zoom = photos_image_view_get_zoom (PHOTOS_IMAGE_VIEW (self->view));
   self->bbox_zoomed.height = (gint) (zoom * self->bbox_source.height + 0.5);
   self->bbox_zoomed.width = (gint) (zoom * self->bbox_source.width + 0.5);
   self->surface = gdk_window_create_similar_surface (window,
@@ -313,7 +312,7 @@ photos_tool_crop_get_active (PhotosToolCrop *self)
 
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->lock_button)))
     {
-      active = gtk_combo_box_get_active (GTK_COMBO_BOX (self->combo_box)) + 1;
+      active = self->list_box_active + 1;
       g_return_val_if_fail (active >= 1, -1);
     }
   else
@@ -810,15 +809,52 @@ photos_tool_crop_active_changed (PhotosToolCrop *self)
 }
 
 
+static void
+photos_tool_crop_list_box_update (PhotosToolCrop *self, GtkListBoxRow *active_row)
+{
+  GtkListBoxRow *row;
+  gint i;
+
+  for (i = 0; (row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->list_box), i)) != NULL; i++)
+    {
+      GtkStyleContext *context;
+      GtkWidget *image;
+      GtkWidget *ratio_label;
+
+      image = GTK_WIDGET (g_object_get_data (G_OBJECT (row), "image"));
+      gtk_widget_set_visible (image, row == active_row);
+
+      ratio_label = GTK_WIDGET (g_object_get_data (G_OBJECT (row), "ratio-label"));
+      context = gtk_widget_get_style_context (ratio_label);
+      if (row == active_row)
+        gtk_style_context_remove_class (context, "dim-label");
+      else
+        gtk_style_context_add_class (context, "dim-label");
+    }
+}
+
+
+static void
+photos_tool_crop_list_box_row_activated (PhotosToolCrop *self, GtkListBoxRow *row)
+{
+  photos_tool_crop_list_box_update (self, row);
+
+  self->list_box_active = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (row));
+  photos_tool_crop_active_changed (self);
+}
+
+
 static void
 photos_tool_crop_set_active (PhotosToolCrop *self, gint active)
 {
-  g_signal_handlers_block_by_func (self->combo_box, photos_tool_crop_active_changed, self);
+  GtkListBoxRow *row;
+
+  g_signal_handlers_block_by_func (self->list_box, photos_tool_crop_list_box_row_activated, self);
   g_signal_handlers_block_by_func (self->lock_button, photos_tool_crop_active_changed, self);
 
   if (active == -1) /* reset */
     {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (self->combo_box), 0);
+      self->list_box_active = 0;
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->lock_button), TRUE);
     }
   else if (active == 0)
@@ -827,7 +863,7 @@ photos_tool_crop_set_active (PhotosToolCrop *self, gint active)
     }
   else if (active > 0)
     {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (self->combo_box), active);
+      self->list_box_active = active - 1;
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->lock_button), TRUE);
     }
   else
@@ -835,55 +871,14 @@ photos_tool_crop_set_active (PhotosToolCrop *self, gint active)
       g_warn_if_reached ();
     }
 
-  g_signal_handlers_unblock_by_func (self->combo_box, photos_tool_crop_active_changed, self);
+  row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->list_box), self->list_box_active);
+  photos_tool_crop_list_box_update (self, row);
+
+  g_signal_handlers_unblock_by_func (self->list_box, photos_tool_crop_list_box_row_activated, self);
   g_signal_handlers_unblock_by_func (self->lock_button, photos_tool_crop_active_changed, self);
 }
 
 
-static void
-photos_tool_crop_process (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  PhotosToolCrop *self = PHOTOS_TOOL_CROP (user_data);
-  GError *error = NULL;
-  PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
-  gfloat zoom;
-  guint active;
-
-  photos_base_item_process_finish (item, res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Unable to process item: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
-
-  zoom = gegl_gtk_view_get_zoom (GEGL_GTK_VIEW (self->view));
-  self->crop_height *= zoom;
-  self->crop_width *= zoom;
-  self->crop_x *= zoom;
-  self->crop_y *= zoom;
-
-  self->crop_aspect_ratio = self->crop_width / self->crop_height;
-  active = photos_tool_crop_find_constraint (self, self->crop_aspect_ratio);
-  photos_tool_crop_set_active (self, (gint) active);
-
-  photos_tool_crop_surface_draw (self);
-
- out:
-  gtk_widget_queue_draw (self->view);
-  g_object_unref (self);
-}
-
-
-static void
-photos_tool_crop_reset_clicked (PhotosToolCrop *self)
-{
-  self->reset = TRUE;
-  photos_tool_crop_set_active (self, -1);
-  g_signal_emit_by_name (self, "hide-requested");
-}
-
-
 static void
 photos_tool_crop_size_allocate (PhotosToolCrop *self, GdkRectangle *allocation)
 {
@@ -909,20 +904,90 @@ photos_tool_crop_size_allocate (PhotosToolCrop *self, GdkRectangle *allocation)
 
 
 static void
-photos_tool_crop_activate (PhotosTool *tool, PhotosBaseItem *item, GeglGtkView *view)
+photos_tool_crop_process (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosToolCrop *self = PHOTOS_TOOL_CROP (user_data);
+  GError *error = NULL;
+  PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
+  gfloat zoom;
+  guint active;
+
+  photos_base_item_process_finish (item, res, &error);
+  if (error != NULL)
+    {
+      g_warning ("Unable to process item: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  photos_tool_crop_surface_create (self);
+
+  zoom = photos_image_view_get_zoom (PHOTOS_IMAGE_VIEW (self->view));
+  self->crop_height *= zoom;
+  self->crop_width *= zoom;
+  self->crop_x *= zoom;
+  self->crop_y *= zoom;
+
+  self->crop_aspect_ratio = self->crop_width / self->crop_height;
+  active = photos_tool_crop_find_constraint (self, self->crop_aspect_ratio);
+  photos_tool_crop_set_active (self, (gint) active);
+
+  photos_tool_crop_surface_draw (self);
+
+  self->size_allocate_id = g_signal_connect_object (self->view,
+                                                    "size-allocate",
+                                                    G_CALLBACK (photos_tool_crop_size_allocate),
+                                                    self,
+                                                    G_CONNECT_SWAPPED);
+
+  self->activated = TRUE;
+  self->reset = FALSE;
+  g_signal_emit_by_name (self, "activated");
+
+ out:
+  g_object_unref (self);
+}
+
+
+static void
+photos_tool_crop_reset_clicked (PhotosToolCrop *self)
+{
+  self->reset = TRUE;
+  photos_tool_crop_set_active (self, -1);
+  g_signal_emit_by_name (self, "hide-requested");
+}
+
+
+static void
+photos_tool_crop_activate (PhotosTool *tool, PhotosBaseItem *item, PhotosImageView *view)
 {
   PhotosToolCrop *self = PHOTOS_TOOL_CROP (tool);
+  GtkListBoxRow *row;
   gboolean got_bbox_source;
   gdouble height = -1.0;
   gdouble width = -1.0;
   gdouble x = -1.0;
   gdouble y = -1.0;
+  gint i;
 
   got_bbox_source = photos_base_item_get_bbox_source (item, &self->bbox_source);
   g_return_if_fail (got_bbox_source);
 
   self->view = GTK_WIDGET (view);
-  photos_tool_crop_surface_create (self);
+
+  for (i = 0; (row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->list_box), i)) != NULL; i++)
+    {
+      GtkWidget *ratio_label;
+      gchar *ratio_text;
+      gdouble constraint_aspect_ratio;
+
+      constraint_aspect_ratio = photos_tool_crop_calculate_aspect_ratio (self, i + 1);
+      /* Translators: this is the aspect ratio of the crop tool */
+      ratio_text = g_strdup_printf (_("%.2f"), constraint_aspect_ratio);
+      ratio_label = GTK_WIDGET (g_object_get_data (G_OBJECT (row), "ratio-label"));
+      gtk_label_set_text (GTK_LABEL (ratio_label), ratio_text);
+      g_free (ratio_text);
+    }
 
   if (photos_base_item_operation_get (item,
                                       "gegl:crop",
@@ -957,17 +1022,19 @@ photos_tool_crop_activate (PhotosTool *tool, PhotosBaseItem *item, GeglGtkView *
     }
   else
     {
+      photos_tool_crop_surface_create (self);
       photos_tool_crop_init_crop (self);
+
+      self->size_allocate_id = g_signal_connect_object (self->view,
+                                                        "size-allocate",
+                                                        G_CALLBACK (photos_tool_crop_size_allocate),
+                                                        self,
+                                                        G_CONNECT_SWAPPED);
+
+      self->activated = TRUE;
+      self->reset = FALSE;
+      g_signal_emit_by_name (self, "activated");
     }
-
-  self->size_allocate_id = g_signal_connect_object (self->view,
-                                                    "size-allocate",
-                                                    G_CALLBACK (photos_tool_crop_size_allocate),
-                                                    self,
-                                                    G_CONNECT_SWAPPED);
-
-  self->activated = TRUE;
-  self->reset = FALSE;
 }
 
 
@@ -984,7 +1051,7 @@ photos_tool_crop_deactivate (PhotosTool *tool)
       GVariantType *parameter_type;
       gfloat zoom;
 
-      zoom = gegl_gtk_view_get_zoom (GEGL_GTK_VIEW (self->view));
+      zoom = photos_image_view_get_zoom (PHOTOS_IMAGE_VIEW (self->view));
 
       /* GEGL needs the parameters to be in device pixels. So, we
        * should multiply the extents of the crop rectangle and the
@@ -1024,10 +1091,10 @@ photos_tool_crop_draw (PhotosTool *tool, cairo_t *cr, GdkRectangle *rect)
   g_return_if_fail (self->activated);
   g_return_if_fail (self->view != NULL);
 
-  x = (gdouble) gegl_gtk_view_get_x (GEGL_GTK_VIEW (self->view));
+  x = (gdouble) photos_image_view_get_x (PHOTOS_IMAGE_VIEW (self->view));
   x = -x;
 
-  y = (gdouble) gegl_gtk_view_get_y (GEGL_GTK_VIEW (self->view));
+  y = (gdouble) photos_image_view_get_y (PHOTOS_IMAGE_VIEW (self->view));
   y = -y;
 
   cairo_save (cr);
@@ -1057,8 +1124,8 @@ photos_tool_crop_left_click_event (PhotosTool *tool, GdkEventButton *event)
 
   self->grabbed = TRUE;
 
-  x = (gdouble) gegl_gtk_view_get_x (GEGL_GTK_VIEW (self->view));
-  y = (gdouble) gegl_gtk_view_get_y (GEGL_GTK_VIEW (self->view));
+  x = (gdouble) photos_image_view_get_x (PHOTOS_IMAGE_VIEW (self->view));
+  y = (gdouble) photos_image_view_get_y (PHOTOS_IMAGE_VIEW (self->view));
   self->event_x_last = event->x + x;
   self->event_y_last = event->y + y;
 
@@ -1094,8 +1161,8 @@ photos_tool_crop_motion_event (PhotosTool *tool, GdkEventMotion *event)
   g_return_val_if_fail (self->activated, GDK_EVENT_PROPAGATE);
   g_return_val_if_fail (self->view != NULL, GDK_EVENT_PROPAGATE);
 
-  x = (gdouble) gegl_gtk_view_get_x (GEGL_GTK_VIEW (self->view));
-  y = (gdouble) gegl_gtk_view_get_y (GEGL_GTK_VIEW (self->view));
+  x = (gdouble) photos_image_view_get_x (PHOTOS_IMAGE_VIEW (self->view));
+  y = (gdouble) photos_image_view_get_y (PHOTOS_IMAGE_VIEW (self->view));
   event_x = event->x + x;
   event_y = event->y + y;
 
@@ -1119,7 +1186,6 @@ photos_tool_crop_dispose (GObject *object)
 {
   PhotosToolCrop *self = PHOTOS_TOOL_CROP (object);
 
-  g_clear_object (&self->model);
   g_clear_object (&self->box);
   g_clear_pointer (&self->surface, (GDestroyNotify) cairo_surface_destroy);
 
@@ -1131,28 +1197,11 @@ static void
 photos_tool_crop_init (PhotosToolCrop *self)
 {
   GApplication *app;
-  GtkCellRenderer *renderer;
   guint i;
 
   app = g_application_get_default ();
   self->crop = g_action_map_lookup_action (G_ACTION_MAP (app), "crop-current");
 
-  self->model = gtk_list_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT);
-
-  for (i = 1; i < G_N_ELEMENTS (CONSTRAINTS); i++)
-    {
-      GtkTreeIter iter;
-
-      gtk_list_store_append (self->model, &iter);
-      gtk_list_store_set (self->model,
-                          &iter,
-                          CONSTRAINT_COLUMN_ASPECT_RATIO, CONSTRAINTS[i].aspect_ratio_type,
-                          CONSTRAINT_COLUMN_NAME, CONSTRAINTS[i].name,
-                          CONSTRAINT_COLUMN_BASIS_HEIGHT, CONSTRAINTS[i].basis_height,
-                          CONSTRAINT_COLUMN_BASIS_WIDTH, CONSTRAINTS[i].basis_width,
-                          -1);
-    }
-
   /* We really need a GtkBox here. A GtkGrid won't work because it
    * doesn't expand the children to fill the full width of the
    * palette.
@@ -1168,9 +1217,52 @@ photos_tool_crop_init (PhotosToolCrop *self)
   gtk_revealer_set_transition_type (GTK_REVEALER (self->revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN);
   g_object_bind_property (self->lock_button, "active", self->revealer, "reveal-child", G_BINDING_DEFAULT);
 
-  self->combo_box = gtk_combo_box_new_with_model (GTK_TREE_MODEL (self->model));
-  gtk_container_add (GTK_CONTAINER (self->revealer), self->combo_box);
-  g_signal_connect_swapped (self->combo_box, "changed", G_CALLBACK (photos_tool_crop_active_changed), self);
+  self->list_box = gtk_list_box_new ();
+  gtk_list_box_set_selection_mode (GTK_LIST_BOX (self->list_box), GTK_SELECTION_NONE);
+  gtk_container_add (GTK_CONTAINER (self->revealer), self->list_box);
+  g_signal_connect_swapped (self->list_box,
+                            "row-activated",
+                            G_CALLBACK (photos_tool_crop_list_box_row_activated),
+                            self);
+
+  for (i = 1; i < G_N_ELEMENTS (CONSTRAINTS); i++)
+    {
+      GtkStyleContext *context;
+      GtkWidget *grid;
+      GtkWidget *image;
+      GtkWidget *label;
+      GtkWidget *ratio_label;
+      GtkWidget *row;
+
+      row = gtk_list_box_row_new ();
+      gtk_container_add (GTK_CONTAINER (self->list_box), row);
+
+      grid = gtk_grid_new ();
+      gtk_widget_set_hexpand (grid, FALSE);
+      gtk_widget_set_margin_bottom (grid, 3);
+      gtk_widget_set_margin_top (grid, 3);
+      gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_HORIZONTAL);
+      gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+      gtk_container_add (GTK_CONTAINER (row), grid);
+
+      label = gtk_label_new (CONSTRAINTS[i].name);
+      gtk_container_add (GTK_CONTAINER (grid), label);
+
+      image = gtk_image_new_from_icon_name (PHOTOS_ICON_OBJECT_SELECT_SYMBOLIC, GTK_ICON_SIZE_INVALID);
+      gtk_widget_set_no_show_all (image, TRUE);
+      gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
+      gtk_container_add (GTK_CONTAINER (grid), image);
+
+      ratio_label = gtk_label_new (NULL);
+      gtk_widget_set_halign (ratio_label, GTK_ALIGN_END);
+      gtk_widget_set_hexpand (ratio_label, TRUE);
+      context = gtk_widget_get_style_context (ratio_label);
+      gtk_style_context_add_class (context, "dim-label");
+      gtk_container_add (GTK_CONTAINER (grid), ratio_label);
+
+      g_object_set_data (G_OBJECT (row), "image", image);
+      g_object_set_data (G_OBJECT (row), "ratio-label", ratio_label);
+    }
 
   photos_tool_crop_set_active (self, -1);
 
@@ -1179,10 +1271,6 @@ photos_tool_crop_init (PhotosToolCrop *self)
   gtk_container_add (GTK_CONTAINER (self->box), self->reset_button);
   g_signal_connect_swapped (self->reset_button, "clicked", G_CALLBACK (photos_tool_crop_reset_clicked), self);
 
-  renderer = gtk_cell_renderer_text_new ();
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (self->combo_box), renderer, TRUE);
-  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (self->combo_box), renderer, "text", CONSTRAINT_COLUMN_NAME);
-
   self->event_x_last = -1.0;
   self->event_y_last = -1.0;
 }
