@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015 Red Hat, Inc.
+ * Copyright © 2015 – 2016 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -103,17 +103,42 @@ photos_tool_filters_create_preview_idle (gpointer user_data)
 
 
 static void
-photos_tool_filters_activate (PhotosTool *tool, PhotosBaseItem *item, GeglGtkView *view)
+photos_tool_filters_activate (PhotosTool *tool, PhotosBaseItem *item, PhotosImageView *view)
 {
   PhotosToolFilters *self = PHOTOS_TOOL_FILTERS (tool);
+  PhotosOperationInstaPreset preset;
 
   if (self->buttons == NULL || self->create_preview_id != 0)
-    return;
+    goto out;
 
   g_clear_object (&self->item);
   self->item = g_object_ref (item);
 
   self->create_preview_id = g_idle_add_full (G_PRIORITY_LOW, photos_tool_filters_create_preview_idle, self, NULL);
+
+  if (photos_base_item_operation_get (item, "photos:insta-filter", "preset", &preset, NULL))
+    {
+      GList *l;
+
+      for (l = self->buttons; l != NULL; l = l->next)
+        {
+          GtkWidget *button = GTK_WIDGET (l->data);
+          GVariant *target_value;
+          PhotosOperationInstaPreset button_preset;
+
+          target_value = gtk_actionable_get_action_target_value (GTK_ACTIONABLE (button));
+          button_preset = (PhotosOperationInstaPreset) g_variant_get_int16 (target_value);
+
+          if (preset == button_preset)
+            {
+              photos_tool_filter_button_set_active (PHOTOS_TOOL_FILTER_BUTTON (button), TRUE);
+              break;
+            }
+        }
+    }
+
+ out:
+  g_signal_emit_by_name (self, "activated");
 }
 
 
@@ -170,35 +195,35 @@ photos_tool_filters_init (PhotosToolFilters *self)
   group = photos_tool_filter_button_get_group (PHOTOS_TOOL_FILTER_BUTTON (button));
   self->buttons = g_list_prepend (self->buttons, button);
 
-  button = photos_tool_filter_button_new (group, _("1977"));
+  button = photos_tool_filter_button_new (group, _("1947"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "app.insta-current");
-  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_1977);
+  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_1947);
   gtk_grid_attach (GTK_GRID (self->grid), button, 1, row, 1, 1);
   self->buttons = g_list_prepend (self->buttons, button);
   row++;
 
-  button = photos_tool_filter_button_new (group, _("Brannan"));
+  button = photos_tool_filter_button_new (group, _("Calistoga"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "app.insta-current");
-  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_BRANNAN);
+  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_CALISTOGA);
   gtk_grid_attach (GTK_GRID (self->grid), button, 0, row, 1, 1);
   self->buttons = g_list_prepend (self->buttons, button);
 
-  button = photos_tool_filter_button_new (group, _("Gotham"));
+  button = photos_tool_filter_button_new (group, _("Mogadishu"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "app.insta-current");
-  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_GOTHAM);
+  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_MOGADISHU);
   gtk_grid_attach (GTK_GRID (self->grid), button, 1, row, 1, 1);
   self->buttons = g_list_prepend (self->buttons, button);
   row++;
 
-  button = photos_tool_filter_button_new (group, _("Gray"));
+  button = photos_tool_filter_button_new (group, _("Caap"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "app.insta-current");
-  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_GRAY);
+  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_CAAP);
   gtk_grid_attach (GTK_GRID (self->grid), button, 0, row, 1, 1);
   self->buttons = g_list_prepend (self->buttons, button);
 
-  button = photos_tool_filter_button_new (group, _("Nashville"));
+  button = photos_tool_filter_button_new (group, _("Hometown"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "app.insta-current");
-  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_NASHVILLE);
+  gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "n", (gint16) PHOTOS_OPERATION_INSTA_PRESET_HOMETOWN);
   gtk_grid_attach (GTK_GRID (self->grid), button, 1, row, 1, 1);
   self->buttons = g_list_prepend (self->buttons, button);
   row++;
